#include "modbus.h"
#include "crc.h"
#define ADDRESS 1

unsigned char xdata RAM_BITS[50];
unsigned int xdata RAM_WORDS[50];

unsigned char xdata response[BUFLEN];

#define MODBUS_RECEIVE         UART1_ready_buf
#define MODBUS_RECEIVE_LEN     UART1_ready_buf_len
#define MODBUS_SEND            UART1_send_buf
#define MODBUS_SEND_INDEX      UART1_send_index
#define MODBUS_SEND_LEN        UART1_send_len

void MODBUS_Init(void)
{
	int i;
	for(i=0;i<50;i++) RAM_BITS[i]=0;
	for(i=0;i<50;i++) RAM_WORDS[i]=0;
}

unsigned int ReadBits(unsigned int address,unsigned int wordcount)
{
	unsigned int crcvalue;
	unsigned char ByteNum;
	unsigned char ByteTemp,CoilVal;
	unsigned char CoilCount=0;
	unsigned char i,j;
	
	ByteNum=wordcount/8;
	if(wordcount%8!=0) ByteNum++;
	
	response[0]=ADDRESS;
	response[1]=1;
	response[2]=ByteNum;		          //
	
	for(i=0;i<ByteNum;i++)
	{
		ByteTemp=0;
		for(j=0;j<8;j++)
		{
			CoilVal=RAM_BITS[address++];//getCoilVal(address++);
			CoilCount++;
			ByteTemp|=CoilVal<<j;
			if(CoilCount>=wordcount) break;
		}
		response[i+3]=ByteTemp;
	}
	crcvalue=CRC16(response,3+ByteNum);
	response[3+ByteNum]=crcvalue;
	response[4+ByteNum]=crcvalue>>8;
	return 5+ByteNum;
}

unsigned int ReadWords(unsigned int address,unsigned int wordcount)
{
	unsigned int crcvalue;
    unsigned int j;
	response[0]=ADDRESS;
	response[1]=3;
	response[2]=(unsigned char)(2*wordcount);
    
    for(j=0;j<wordcount;j++)
    {
        response[3+2*j]=RAM_WORDS[address+j]>>8;
        response[4+2*j]=RAM_WORDS[address+j];
    }
	crcvalue=CRC16(response,2*wordcount+3);
	response[2*wordcount+3]=crcvalue;
	response[2*wordcount+4]=crcvalue>>8;
	return 2*wordcount+5;
}

unsigned int WriteWords(unsigned int address,unsigned int wordcount)
{
    unsigned int crcvalue;
    unsigned int j;
    for(j=0;j<wordcount;j++)
    {
        RAM_WORDS[address+j]=MODBUS_RECEIVE[7+2*j]*256+MODBUS_RECEIVE[8+2*j];
    }
    for(j=0;j<6;j++)
    {
        response[j]=MODBUS_RECEIVE[j];
    }
	crcvalue=CRC16(response,6);
	response[6]=crcvalue;
	response[7]=crcvalue>>8;
	return 8;     
}


void Modbus_Process(void)
{
	unsigned char i;
	unsigned int address;
    unsigned int wordcount;
	unsigned int retlen;
	
	if(UART1_ready_buf_ok)
	{
		UART1_ready_buf_ok=0;
		
		if(MODBUS_RECEIVE[0] == ADDRESS)
			{
				if(CRC16_Check(MODBUS_RECEIVE,MODBUS_RECEIVE_LEN))
				{
		//			Run_led=!Run_led;
					address=MODBUS_RECEIVE[2]*256+MODBUS_RECEIVE[3];
					wordcount=MODBUS_RECEIVE[4]*256+MODBUS_RECEIVE[5];
					
					switch(MODBUS_RECEIVE[1])
					{
						case 0x01://λ
							retlen=ReadBits(address,wordcount);
							break;
						
						case 0x03://ֱ
							retlen=ReadWords(address,wordcount);
							break;
						
						case 0x05://дλ
							RAM_BITS[address]=!!MODBUS_RECEIVE[4];
							retlen=MODBUS_RECEIVE_LEN;
							for(i=0;i<retlen;i++)response[i]=MODBUS_RECEIVE[i];
							break;
						
						case 0x06://дֱ
							RAM_WORDS[address]=MODBUS_RECEIVE[4]*256+MODBUS_RECEIVE[5];
							retlen=MODBUS_RECEIVE_LEN;
							for(i=0;i<retlen;i++)response[i]=MODBUS_RECEIVE[i];
							break;
						
						case 0x10://дֱ
							retlen=WriteWords(address,wordcount);
							break;
						
						default:
							retlen=0;
							break;
					}

					//response
					for(i=0;i<retlen;i++)
						MODBUS_SEND[i]=response[i];
					MODBUS_SEND_INDEX=0;
					MODBUS_SEND_LEN=retlen;
					//S2CON |= S2TI;
					TI = 1;	
				}				
			}

	}
}

